@@ -12,18 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Utility for determining what AWS account and identity you're using."""
+
 from __future__ import print_function
 
 import argparse
 from collections import namedtuple
 import json
 import sys
+import os
 import traceback
 
-import boto3
+import botocore
+import botocore.session
 from botocore.exceptions import ClientError
 
-__version__ = '0.2.2'
+__version__ = '1.1.0'
 
 WhoamiInfo = namedtuple('WhoamiInfo', [
     'Account',
@@ -60,9 +64,17 @@ def main():
         parser.exit()
 
     try:
-        session = boto3.Session(profile_name=args.profile)
+        session = botocore.session.Session(profile=args.profile)
 
-        whoami_info = whoami(session)
+        disable_account_alias = os.environ.get('AWS_WHOAMI_DISABLE_ACCOUNT_ALIAS', '')
+        if disable_account_alias.lower() in ['', '0', 'false']:
+            disable_account_alias = False
+        elif disable_account_alias.lower() in ['1', 'true']:
+            disable_account_alias = True
+        else:
+            disable_account_alias = disable_account_alias.split(',')
+
+        whoami_info = whoami(session=session, disable_account_alias=disable_account_alias)
 
         if args.json:
             print(json.dumps(whoami_info._asdict()))
@@ -93,13 +105,26 @@ def format_whoami(whoami_info):
     max_len = max(len(l[0]) for l in lines)
     return '\n'.join("{}{}".format(l[0].ljust(max_len), l[1]) for l in lines)
 
-def whoami(session=None):
-    session = session or boto3.Session()
+def whoami(session=None, disable_account_alias=False):
+    """Return a WhoamiInfo namedtuple.
+
+    Args:
+        session: An optional boto3 or botocore Session
+        disable_account_alias (bool): Disable checking the account alias
+
+    Returns:
+        WhoamiInfo: Data on the current IAM principal, account, and region.
+
+    """
+    if session is None:
+        session = botocore.session.get_session()
+    elif hasattr(session, '_session'):
+        session = session._session
 
     data = {}
-    data['Region'] = session.region_name
+    data['Region'] = session.get_config_variable('region')
 
-    response = session.client('sts').get_caller_identity()
+    response = session.create_client('sts').get_caller_identity()
 
     for field in ['Account', 'Arn', 'UserId']:
         data[field] = response[field]
@@ -113,14 +138,24 @@ def whoami(session=None):
         data['RoleSessionName'] = None
 
     data['AccountAliases'] = []
-    try:
-        #pedantry
-        paginator = session.client('iam').get_paginator('list_account_aliases')
-        for response in paginator.paginate():
-            data['AccountAliases'].extend(response['AccountAliases'])
-    except ClientError as e:
-        if e.response.get('Error', {}).get('Code') != 'AccessDenied':
-            raise
+    if not isinstance(disable_account_alias, bool):
+        for value in disable_account_alias:
+            if data['Account'].startswith(value) or data['Account'].endswith(value):
+                disable_account_alias = True
+                break
+            fields = ['Name', 'Arn', 'RoleSessionName']
+            if any(value == data[field] for field in fields):
+                disable_account_alias = True
+                break
+    if not disable_account_alias:
+        try:
+            #pedantry
+            paginator = session.create_client('iam').get_paginator('list_account_aliases')
+            for response in paginator.paginate():
+                data['AccountAliases'].extend(response['AccountAliases'])
+        except ClientError as e:
+            if e.response.get('Error', {}).get('Code') != 'AccessDenied':
+                raise
 
     return WhoamiInfo(**data)
 
